#baseN_converter
# Python script which converts a decimal number to a base N equivalent
# LAST UPDATED by Andy French May 2018

#Function which defines a list of symbols corresponding to the possible digits
#of a base N
def base_symbols(N):

    #Truncate number of symbols of N is too high
    if N>62:
        N = 62

    #Initialize list of symbols
    s = [""]*N

    #Digit symbols
    symbols = ["0","1","2","3","4","5","6","7","8","9",
                "A","B","C","D","E","F","G","H","I","J","K","L","M","N","O",
                "P","Q","R","S","T","U","V","W","X","Y","Z",
                "a","b","c","d","e","f","g","h","i","j","k","l","m","n","o",
                "p","q","r","s","t","u","v","w","x","y","z"]

    #Assign symbolsS   
    for n in range(0,N):
        s[n] = symbols[n]

    #Set function output
    return s

#

#Function which determines base N symbol representation of integer x
#Note if x is a decimal it will be converted to an integer first
def baseN(x,s):

    #Convert input to integer
    y = abs(x)

    #Initialize list of digit strings
    digits = []

    #Iterate y, subtracting remainders following division by N
    N = len(s)
    while y!=0:
        digits = [ s[ int(y%N) ] ] + digits
        y = ( y - y%N )/N

    #Transform digits to a text string
    x_str_base_N = "".join(digits)

    #Set function output
    return x_str_base_N

#

#Function which returns base10 integer given a string of base N
def base10_from_N(x_str,N):

    #Get base symbols
    s = base_symbols(N)

    #Construct a list array of the individual symbols in x_str
    x = list(x_str)

    #Step through list and construct base10 integer y
    y = 0
    for i in range(len(x)):
        #Find base 10 digit of digit symbol
        try: 
            base10_digit = s.index( x[i] )
        except:
            y = -1
            print(" ERROR! Integer digit string must have symbols belonging to specified base \n ")
            break

        #Add to y
        y = y + base10_digit * ( N**( len(x) - 1 - i ) )

    #Set output
    return y

#

#Test code
import math
print("\n Base N integer converter by Andy French. May 2018 \n")

#Choose base of input
B = input( " Choose base B for input number n : B = ")
if B=="":
    B = 62
    print( " Base B = " + str(B) )
else:
    try:
        B = int(B)
    except:
        B = 2
        print( " Base B = " + str(B) )
    if B>62:
        B = 62
        print( " Base B = " + str(B) )
    elif B<2:
        B = 2
        print( " Base B = " + str(B) )

#Choose input number (base B) and convert to base 10 integer
n = input( "\n Choose input (base B = " + str(B) + ") n =")
if (n=="") & ( B==10 ):
    n = math.pi*1E30
elif (n=="") & ( B==62 ):
    n = "AndyFrench"
    print( " base 62 n = ",n)
    n = base10_from_N(n,B)
else:
    n = base10_from_N(n,B)
if n==-1:
    n = 12345
print( " base 10 n = ",str(n))

#Choose output number (base B)
N = input( "\n Choose base N for output: Base N = ")
if N=="":
    N = 60
    print( " N = " + str(N) )
else:
    N = int(N)
s = base_symbols(N)
print(" Base " + str(N) + " symbols are: ")
print(s)
digits = baseN(n,s)
print("\n n = " + digits + " in base " + str(N) + "\n n = " + str( base10_from_N(digits,N) ) + " in base 10")
input("\n Press return to close ")

#End of code
