#differentiate
# Example Python code to fit a cubic spline to a set of x,y samples and hence
# form an approximation to its derivative

#Import Python numeric and plotting libraries
import numpy as np
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
from scipy.interpolate import CubicSpline as cspline

#

#Generate x,y data to differentiate
pi = float(np.pi)
xmin = 0
xmax = 2*2*pi

#Exact
x = np.linspace(xmin,xmax,1000)
y = np.sin(x)
dy_by_dx = np.cos(x)

#Samples to illustrate cubic splines
xs = np.linspace(xmin,xmax,20)
ys = np.sin(xs)

#Generate cubic spline and evaluate at xs
cs = cspline( xs, ys )
ys_cs = cs(xs)

#Generate cubic spline and evaluate derivative at x
cs = cspline( x, y )
dy_by_dx_cs = cs(x,1)

#Create figure and axis
fig = plt.figure(figsize=(10, 8))

#Plot y vs x
fsize = 18
title_str = "Cubic-Spline fit of y = sin(x)"
plt.plot( x,y,'-',label = "y = sin(x)" , color = [1,0,0] )
plt.plot( xs,ys,'.',label = "Samples", color = [0,1,0] )
plt.plot( xs,ys_cs,'--',label = "Cubic spline", color = [0,0,1] )
plt.xlabel( "x", size=fsize );
plt.ylabel( "y", size=fsize );
plt.xticks( fontsize = fsize )
plt.yticks( fontsize = fsize )
plt.title( title_str, size=fsize )
plt.legend( fontsize = fsize,loc = "upper right" )
plt.xlim( xmin, xmax )
plt.grid(True)

#Export PNG file
fig.savefig('y vs x cubic spline.png', dpi=300)

#Wait for user input to close terminal window
input("\n Press return key to close ...")

#End of code
